@@ -1,3 +1,7 @@
+/* Copyright 2017-2023. AG Uecker. University Medical Center Göttingen.
+ * All rights reserved. Use of this source code is governed by
+ * a BSD-style license which can be found in the LICENSE file.
+ */
 
 #include <stdio.h>
 #include <assert.h>
@@ -13,6 +17,12 @@
 #include "misc/mmio.h"
 #include "misc/opts.h"
 #include "misc/png.h"
+#if 0
+#include "misc/io.h"
+#else
+extern void io_reserve_input(const char* name);
+extern void io_unregister(const char* name);
+#endif
 
 #include "draw.h"
 
@@ -25,7 +35,7 @@
 #endif
 
 
-static void export_images(const char* output_prefix, int xdim, int ydim, float windowing[2], float zoom, enum mode_t mode, enum flip_t flip, enum interp_t interpolation, const long dims[DIMS], const complex float* idata);
+static void export_images(const char* output_prefix, int xdim, int ydim, float windowing[2], bool absolute_windowing, float zoom, enum mode_t mode, enum flip_t flip, enum interp_t interpolation, const long dims[DIMS], const complex float* idata);
 
 
 static const char help_str[] = "Export images to png.";
@@ -47,6 +57,7 @@ int main(int argc, char* argv[argc])
 	int xdim = 0;
 	int ydim = 0;
 	float windowing[2] = {0.f, 1.f};
+	bool absolute_windowing = false;
 	enum mode_t mode = MAGN;
 	float zoom =2.f;
 	enum flip_t flip = OO;
@@ -60,6 +71,7 @@ int main(int argc, char* argv[argc])
 		OPT_SELECT('P', enum mode_t, &mode, PHSE, 		"phase"),
 		OPT_SELECT('Y', enum mode_t, &mode, PHSE_MYGBM, 	"phase MYGBM"),
 		OPT_SELECT('R', enum mode_t, &mode, REAL, 		"real"),
+		OPT_SELECT('T', enum mode_t, &mode, MAGN_TURBO,		"magnitude turbo"),
 		OPT_SELECT('F', enum mode_t, &mode, FLOW, 		"flow"),
 	};
 
@@ -72,7 +84,9 @@ int main(int argc, char* argv[argc])
 
 	struct opt_s interpopt[] = {
 		OPT_SELECT('L', enum interp_t, &interpolation, NLINEAR, 		"n-linear interpolation"),
+		OPT_SELECT('M', enum interp_t, &interpolation, NLINEARMAG, 		"n-linear interpolation on the magnitude"),
 		OPT_SELECT('N', enum interp_t, &interpolation, NEAREST, 		"nearest neighbor interpolation"),
+		OPT_SELECT('C', enum interp_t, &interpolation, LIINCO, 			"line-integral convolution"),
 	};
 
 	const struct opt_s opts[] = {
@@ -80,6 +94,7 @@ int main(int argc, char* argv[argc])
 		OPT_INT('y', &ydim, "ydim", "output ydim (default: 0). If both are zero, first two non-singleton dims are used."),
 		OPT_FLOAT('l', &windowing[0], "l", "lower windowing value"),
 		OPT_FLOAT('u', &windowing[1], "u", "upper windowing value"),
+		OPT_SET('A', &absolute_windowing, "use absolute windowing"),
 		OPT_FLOAT('z', &zoom, "z", "zoom factor (default: 2) "),
 		OPT_SUBOPT('C', "cmap", "colormap. -Ch for help.", ARRAY_SIZE(modeopt), modeopt),
 		OPT_SUBOPT('F', "flip", "flip. -Fh for help.", ARRAY_SIZE(flipopt), flipopt),
@@ -89,9 +104,10 @@ int main(int argc, char* argv[argc])
 
 	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
-	assert(    (windowing[0] >= 0.f)
-		&& (windowing[1] <= 1.f)
-		&& (windowing[0] < windowing[1]));
+	if (!absolute_windowing)
+		assert(    (windowing[0] >= 0.f)
+			&& (windowing[1] <= 1.f)
+			&& (windowing[0] < windowing[1]));
 
 	assert((0 <= xdim) && (xdim < DIMS));
 	assert((0 <= ydim) && (ydim < DIMS));
@@ -100,6 +116,7 @@ int main(int argc, char* argv[argc])
 	 * If the filename ends in ".hdr", ".cfl" or just "." (from
 	 * tab-completion), just replace the "." with a \0-character.
 	 */
+	io_unregister(in_file);
 	char* dot = strrchr(in_file, '.');
 
 	if (   (NULL != dot)
@@ -108,6 +125,7 @@ int main(int argc, char* argv[argc])
 		|| !strcmp(dot, ".")))
 		*dot = '\0';
 
+	io_reserve_input(in_file);
 
 	long dims[DIMS];
 	complex float* idata = load_cfl(in_file, DIMS, dims);
@@ -122,7 +140,7 @@ int main(int argc, char* argv[argc])
 			*ext = '\0';
 	}
 
-	export_images(out_prefix, xdim, ydim, windowing, zoom, mode, flip, interpolation, dims, idata);
+	export_images(out_prefix, xdim, ydim, windowing, absolute_windowing, zoom, mode, flip, interpolation, dims, idata);
 
 
 	unmap_cfl(DIMS, dims, idata);
@@ -131,25 +149,24 @@ int main(int argc, char* argv[argc])
 }
 
 
-
 /**
  * Convert flat index to pos
  *
  */
-static void unravel_index(unsigned int D, long pos[D], const long dims[D], long index)
+static void unravel_index(int D, long pos[D], unsigned long flags, const long dims[D], long index)
 {
-	for (unsigned int d = 0; d < D; ++d) {
+	long ind = index;
+	for (int d = 0; d < D; ++d) {
 
-		if (1 == dims[d])
+		if (!MD_IS_SET(flags, d))
 			continue;
 
-		pos[d] = index % dims[d];
-		index /= dims[d];
+		pos[d] = ind % dims[d];
+		ind /= dims[d];
 	}
 }
 
-
-void export_images(const char* output_prefix, int xdim, int ydim, float windowing[2], float zoom, enum mode_t mode, enum flip_t flip, enum interp_t interpolation, const long dims[DIMS], const complex float* idata)
+void export_images(const char* output_prefix, int xdim, int ydim, float windowing[2], bool absolute_windowing, float zoom, enum mode_t mode, enum flip_t flip, enum interp_t interpolation, const long dims[DIMS], const complex float* idata)
 {
 	if (xdim == ydim) {
 
@@ -167,83 +184,71 @@ void export_images(const char* output_prefix, int xdim, int ydim, float windowin
 	}
 
 	double max = 0.;
-	for (long j = 0; j < md_calc_size(DIMS, dims); j++)
-		if (max < cabsf(idata[j]))
-			max = cabsf(idata[j]);
 
-	if (0. == max)
+	if (absolute_windowing) {
+
 		max = 1.;
+	} else {
+
+		for (long j = 0; j < md_calc_size(DIMS, dims); j++)
+			if (max < cabsf(idata[j]))
+				max = cabsf(idata[j]);
+
+		if (0. == max)
+			max = 1.;
+	}
 
 	int rgbw = dims[xdim] * zoom;
 	int rgbh = dims[ydim] * zoom;
 	int rgbstr = 4 * rgbw;
-	unsigned char* rgb = xmalloc(rgbh * rgbstr);
-
-	complex float* buf = xmalloc(rgbh * rgbw * sizeof(complex float));
 
 	// loop over all dims other than xdim and ydim
 	long loopdims[DIMS];
-	unsigned long loopflags = (MD_BIT(xdim)|MD_BIT(ydim));
-	md_select_dims(DIMS, ~loopflags, loopdims, dims);
+	unsigned long imflags = (MD_BIT(xdim)|MD_BIT(ydim));
+	md_select_dims(DIMS, ~imflags, loopdims, dims);
 
-	debug_printf(DP_DEBUG3, "flags: %lu\nloopdims: ", loopflags);
+	debug_printf(DP_DEBUG3, "imflags: %lu\nloopdims: ", imflags);
 	debug_print_dims(DP_DEBUG3, DIMS, loopdims);
 
 
 
-	long pos[DIMS] = { [0 ... DIMS - 1] = 0  };
 	long strs[DIMS];
 	md_calc_strides(DIMS, strs, dims, sizeof(complex float));
 
-	for (unsigned long d = 0l; d < md_calc_size(DIMS, loopdims); ++d){
 
-		unravel_index(DIMS, pos, loopdims, d);
+#pragma omp parallel for
+	for (unsigned long d = 0l; d < md_calc_size(DIMS, loopdims); ++d) {
+
+		long pos[DIMS] = { [0 ... DIMS - 1] = 0  };
+		unravel_index(DIMS, pos, ~0L, loopdims, d);
 
 		debug_printf(DP_DEBUG3, "\ti: %lu\n\t", d);
 		debug_print_dims(DP_DEBUG3, DIMS, pos);
 
 		// Prepare output filename
-		static const char* spec = "xyzcmnopqsfrtuvw";
-		int len = 0;
-
-		len += snprintf(NULL, 0, "%s_", output_prefix);
-
-		for (unsigned int i = 0; i < DIMS; i++)
-			if (1 != loopdims[i])
-				len += snprintf(NULL, 0, "%c%04ld", spec[i], pos[i]);
-
-		len += snprintf(NULL, 0, ".png");
-		len++;
-
-		char* name = xmalloc(len);
-		int off = 0;
-
-		off += snprintf(name + off, len - off, "%s_", output_prefix);
-
-		for (unsigned int i = 0; i < DIMS; i++)
-			if (1 != loopdims[i])
-				off += snprintf(name + off, len - off, "%c%04ld", spec[i], pos[i]);
-
-		off += snprintf(name + off, len - off, ".png");
+		char* name = construct_filename_view(DIMS, loopdims, pos, output_prefix, "png");
 
 		debug_printf(DP_DEBUG2, "\t%s\n", name);
 
+		complex float* buf = xmalloc(rgbh * rgbw * sizeof(complex float));
+
 		update_buf(xdim, ydim, DIMS, dims, strs, pos,
 			   flip, interpolation, zoom, zoom, false,
 			   rgbw, rgbh, idata, buf);
 
+		unsigned char* rgb = xmalloc(rgbh * rgbstr);
+
 		draw(rgbw, rgbh, rgbstr, (unsigned char(*)[rgbw][rgbstr / 4][4])rgb,
 			mode, 1. / max, windowing[0], windowing[1], 0,
 			rgbw, buf);
 
+		xfree(buf);
+
 		if (0 != png_write_bgr32(name, rgbw, rgbh, 0, rgb))
 			error("Error: writing image file.\n");
 
+		xfree(rgb);
 		xfree(name);
 	}
-
-// 	free(source);
-	free(buf);
-	free(rgb);
 }
 
