@@ -9,6 +9,7 @@
 #include <math.h>
 #include <complex.h>
 #include <assert.h>
+#include <stdio.h>
 
 #include "misc/misc.h"
 
@@ -46,15 +47,29 @@ static void trans_magnitude(double rgb[3], double a, double b, complex double va
 	rgb[2] *= magn;
 }
 
+static void interpolate_cmap(double rgb[3], double x, const double cmap[256][3])
+{
+	int a = x * 255;
+	int b = MIN(255, a + 1);
+
+	double f = x * 255. - a;
+
+	for (int i = 0; i < 3; ++i)
+		rgb[i] *= cmap[a][i] + f * (cmap[b][i] - cmap[a][i]);
+}
+
 static void trans_magnitude_viridis(double rgb[3], double a, double b, complex double value)
 {
 	double magn = window(a, b, cabs(value));
 
-	int subscript = magn * 255.;
+	interpolate_cmap(rgb, magn, viridis);
+}
 
-	rgb[0] *= viridis[subscript][0];
-	rgb[1] *= viridis[subscript][1];
-	rgb[2] *= viridis[subscript][2];
+static void trans_magnitude_turbo(double rgb[3], double a, double b, complex double value)
+{
+	double magn = window(a, b, cabs(value));
+
+	interpolate_cmap(rgb, magn, turbo);
 }
 
 static void trans_real(double rgb[3], double a, double b, complex double value)
@@ -83,13 +98,10 @@ static void trans_phase_MYGBM(double rgb[3], double a, double b, complex double
 
 	if (isfinite(arg)) {
 
-		int subscript = (arg + M_PI) / 2. / M_PI * 255.;
-
-		assert((0 <= subscript) && (subscript <= 255));
+		double val = (arg + M_PI) / 2. / M_PI;
+		assert((0.0 <= val) && (val <= 1.0));
 	
-		rgb[0] *= cyclic_mygbm[subscript][0];
-		rgb[1] *= cyclic_mygbm[subscript][1];
-		rgb[2] *= cyclic_mygbm[subscript][2];
+		interpolate_cmap(rgb, val, cyclic_mygbm);
 	}
 }
 
@@ -126,6 +138,13 @@ static complex float int_nlinear(int N, const float x[N], const long strs[N], co
 		           +       x[N - 1]  * int_nlinear(N - 1, x, strs, in + strs[N - 1]));
 }
 
+static complex float int_nlinearmag(int N, const float x[N], const long strs[N], const complex float* in)
+{
+	return (0 == N) ? cabsf(in[0])
+			: (  (1. - x[N - 1]) * int_nlinearmag(N - 1, x, strs, in + 0)
+		           +       x[N - 1]  * int_nlinearmag(N - 1, x, strs, in + strs[N - 1]));
+}
+
 
 static complex float int_nearest(int N, const float x[N], const long strs[N], const complex float* in)
 {
@@ -199,6 +218,9 @@ complex float sample(int N, const float pos[N], const long dims[N], const long s
 	case NLINEAR:
 		return int_nlinear(D, rem, strs2, (const complex float*)(((char*)in) + off0));
 
+	case NLINEARMAG:
+		return int_nlinearmag(D, rem, strs2, (const complex float*)(((char*)in) + off0));
+
 	case NEAREST:
 		return int_nearest(D, rem, strs2, (const complex float*)(((char*)in) + off0));
 
@@ -237,12 +259,11 @@ complex float lic_sample(int N, const float pos[N], const long dims[N], const lo
 	complex float out = 0.;
 
 	float pos1[N];
-
-	complex float val = sample(N, pos1, dims, strs, NLINEAR, in);
-
 	for (int i = 0; i < N; i++)
 		pos1[i] = pos[i];
 
+
+	complex float val = sample(N, pos1, dims, strs, NLINEAR, in);
 	for (int i = 0; i < L; i++) {
 
 		complex float a = sample(N, pos1, dims, strs, NLINEAR, in);
@@ -292,6 +313,7 @@ extern void resample(int X, int Y, long str, complex float* buf,
 	int N, const double pos[N], const double dx[N], const double dy[N], 
 	const long dims[N], const long strs[N], enum interp_t interpolation, const complex float* in)
 {
+#pragma omp parallel for collapse(2)
 	for (int x = 0; x < X; x++) {
 		for (int y = 0; y < Y; y++) {
 
@@ -327,26 +349,37 @@ extern void draw(int X, int Y, int rgbstr, unsigned char (*rgbbuf)[Y][rgbstr / 4
 	enum mode_t mode, float scale, float winlow, float winhigh, float phrot,
 	long str, const complex float* buf)
 {
+#pragma omp parallel for collapse(2)
 	for (int x = 0; x < X; x++) {
 		for (int y = 0; y < Y; y++) {
 
-			complex float val = scale * buf[str * y + x];
-
-			val *= cexpf(1.i * phrot);
-
 			double rgb[3] = { 1., 1., 1. };
 
-			switch (mode) {
+			complex float val = scale * buf[str * y + x];
 
-			case MAGN: trans_magnitude(rgb, winlow, winhigh, val); break;
-			case MAGN_VIRIDS: trans_magnitude_viridis(rgb, winlow, winhigh, val); break;
-			case PHSE: trans_phase(rgb, winlow, winhigh, val); break;
-			case PHSE_MYGBM: trans_phase_MYGBM(rgb, winlow, winhigh, val); break;
-			case CMPL: trans_complex(rgb, winlow, winhigh, val); break;
-			case CMPL_MYGBM: trans_complex_MYGBM(rgb, winlow, winhigh, val); break;
-			case REAL: trans_real(rgb, winlow, winhigh, val); break;
-			case FLOW: trans_flow(rgb, winlow, winhigh, val); break;
-			default: assert(0);
+			if (isfinite(crealf(val)) && isfinite(cimagf(val))) {
+
+				val *= cexpf(1.i * phrot);
+
+				switch (mode) {
+
+				case MAGN: trans_magnitude(rgb, winlow, winhigh, val); break;
+				case MAGN_VIRIDS: trans_magnitude_viridis(rgb, winlow, winhigh, val); break;
+				case PHSE: trans_phase(rgb, winlow, winhigh, val); break;
+				case PHSE_MYGBM: trans_phase_MYGBM(rgb, winlow, winhigh, val); break;
+				case CMPL: trans_complex(rgb, winlow, winhigh, val); break;
+				case CMPL_MYGBM: trans_complex_MYGBM(rgb, winlow, winhigh, val); break;
+				case REAL: trans_real(rgb, winlow, winhigh, val); break;
+				case MAGN_TURBO: trans_magnitude_turbo(rgb, winlow, winhigh, val); break;
+				case FLOW: trans_flow(rgb, winlow, winhigh, val); break;
+				default: assert(0);
+				}
+
+			} else {
+
+				rgb[0] = 0.;
+				rgb[1] = 0.;
+				rgb[2] = 0.;
 			}
 
 			(*rgbbuf)[y][x][0] = 255. * rgb[2];
@@ -402,6 +435,8 @@ void update_buf(long xdim, long ydim, int N, const long dims[N], const long strs
 			dy[ydim] *= -1.;
 		}
 	}
+	assert(xdim < N);
+	assert(ydim < N);
 
 	dx[xdim] = dx[xdim] / xzoom;
 	dy[ydim] = dy[ydim] / yzoom;
@@ -503,3 +538,35 @@ extern void draw_plot(int X, int Y, int rgbstr, unsigned char (*rgbbuf)[Y][rgbst
 	}
 }
 
+
+static const char* spec = "xyzcmnopqsfrtuvw";
+
+char* construct_filename_view(unsigned int D, const long loopdims[D], const long pos[D], const char* prefix, const char* ext)
+{
+	// Prepare output filename
+
+	int len = 0;
+
+	len += snprintf(NULL, 0, "%s", prefix);
+
+	for (unsigned int i = 0; i < D; i++)
+		if (1 != loopdims[i])
+			len += snprintf(NULL, 0, "_%c%04ld", spec[i], pos[i]);
+
+	len += snprintf(NULL, 0, ".%s", ext);
+	len++;
+
+	char* name = xmalloc(len);
+	int off = 0;
+
+	off += snprintf(name + off, len - off, "%s", prefix);
+
+	for (unsigned int i = 0; i < D; i++)
+		if (1 != loopdims[i])
+			off += snprintf(name + off, len - off, "_%c%04ld", spec[i], pos[i]);
+
+	off += snprintf(name + off, len - off, ".%s", ext);
+
+	return name;
+}
+
