@@ -1,9 +1,6 @@
-/* Copyright 2015-2016. Martin Uecker.
+/* Copyright 2015-2023. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
- *
- * Author:
- *	2015-2016 Martin Uecker <martin.uecker@med.uni-goettinge.de>
  */
 
 #include <complex.h>
@@ -17,6 +14,12 @@
 #include "misc/misc.h"
 #include "misc/mmio.h"
 #include "misc/opts.h"
+#if 0
+#include "misc/io.h"
+#else
+extern void io_reserve_input(const char* name);
+extern void io_unregister(const char* name);
+#endif
 
 #include "view.h"
 
@@ -27,6 +30,7 @@ static const char help_str[] = "View images.";
 
 int main(int argc, char* argv[argc])
 {
+	gtk_disable_setlocale();
 	gtk_init(&argc, &argv);
 
 	long count;
@@ -37,10 +41,11 @@ int main(int argc, char* argv[argc])
 		ARG_TUPLE(true, &count, 1, { OPT_INFILE, sizeof(char*), &in_files, "image" }),
 	};
 
-
+	bool absolute_windowing = false;
 
 	const struct opt_s opts[] = {
 
+		OPT_SET('a', &absolute_windowing, "Use absolute windowing"),
 	};
 
 	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
@@ -60,6 +65,7 @@ int main(int argc, char* argv[argc])
 		 * a file manager.
 		 */
 
+		io_unregister(in_files[i]);
 		char* dot = strrchr(in_files[i], '.');
 
 		if ((NULL != dot) && (	 !strcmp(dot, ".cfl") 
@@ -67,12 +73,13 @@ int main(int argc, char* argv[argc])
 				      || !strcmp(dot, ".")))
 			*dot = '\0';
 
+		io_reserve_input(in_files[i]);
 		complex float* x = load_cfl(in_files[i], DIMS, dims);
 
 
 
 		// FIXME: we never delete them
-		struct view_s* v2 = window_new(in_files[i], NULL, dims, x);
+		struct view_s* v2 = window_new(in_files[i], NULL, dims, x, absolute_windowing);
 
 		// If multiple files are passed on the commandline, add them to window
 		// list. This enables sync of windowing and so on...
