@@ -5,7 +5,7 @@
 
 struct view_s;
 
-extern struct view_s* window_new(const char* name, const long pos[DIMS], const long dims[DIMS], const complex float* x);
+extern struct view_s* window_new(const char* name, const long pos[DIMS], const long dims[DIMS], const complex float* x, _Bool absolute_windowing);
 
 extern void view_add_geometry(struct view_s* v, unsigned long flags, const float (*geom)[3][3]);
 extern void window_connect_sync(struct view_s* a, struct view_s* b);
